@@ -568,7 +568,7 @@ bool AmazonS3Download::SendRequest(off_t offset, size_t size) {
 		formatstr(range, "bytes=%lld-%lld", static_cast<long long int>(offset),
 				  static_cast<long long int>(offset + size - 1));
 		headers["Range"] = range.c_str();
-		this->expectedResponseCode = 206;
+		this->expectedResponseCode = {206};
 	}
 	if (size && m_buffer) {
 		m_buffer_view = std::string_view(m_buffer, size);
